@@ -32,8 +32,14 @@
 #include "osinfo-db-util.h"
 
 #define LATEST_URI "https://db.libosinfo.org/latest.json"
+#define NIGHTLY_URI "https://db.libosinfo.org/nightly.json"
 #define VERSION_FILE "VERSION"
 
+#if SOUP_MAJOR_VERSION < 3
+# define soup_message_get_status(message) message->status_code
+# define soup_message_get_response_headers(message) message->response_headers
+#endif
+
 const char *argv0;
 static SoupSession *session = NULL;
 
@@ -171,15 +177,15 @@ osinfo_db_import_download_file(const gchar *source)
 
     stream = soup_session_send(session, message, NULL, &err);
     if (stream == NULL ||
-        !SOUP_STATUS_IS_SUCCESSFUL(message->status_code)) {
+        !SOUP_STATUS_IS_SUCCESSFUL(soup_message_get_status(message))) {
         g_printerr("Could not access %s: %s\n",
                    source,
                    err != NULL ? err->message :
-                                 soup_status_get_phrase(message->status_code));
+                                 soup_status_get_phrase(soup_message_get_status(message)));
         goto cleanup;
     }
 
-    content_size = soup_message_headers_get_content_length(message->response_headers);
+    content_size = soup_message_headers_get_content_length(soup_message_get_response_headers(message));
     content = g_malloc0(content_size);
 
     if (!g_input_stream_read_all(stream, content, content_size, NULL, NULL, &err)) {
@@ -227,8 +233,9 @@ static gboolean osinfo_db_get_installed_version(GFile *dir,
     return TRUE;
 }
 
-static gboolean osinfo_db_get_latest_info(gchar **version,
-                                          gchar **url)
+static gboolean osinfo_db_get_info(const gchar *from_url,
+                                   gchar **version,
+                                   gchar **url)
 {
     g_autoptr(SoupMessage) message = NULL;
     g_autoptr(GInputStream) stream = NULL;
@@ -244,26 +251,26 @@ static gboolean osinfo_db_get_latest_info(gchar **version,
     if (session == NULL)
         return FALSE;
 
-    message = soup_message_new("GET", LATEST_URI);
+    message = soup_message_new("GET", from_url);
     if (message == NULL)
         return FALSE;
 
     stream = soup_session_send(session, message, NULL, &err);
     if (stream == NULL ||
-        !SOUP_STATUS_IS_SUCCESSFUL(message->status_code)) {
+        !SOUP_STATUS_IS_SUCCESSFUL(soup_message_get_status(message))) {
         g_printerr("Could not access %s: %s\n",
-                   LATEST_URI,
+                   from_url,
                    err != NULL ? err->message :
-                                 soup_status_get_phrase(message->status_code));
+                                 soup_status_get_phrase(soup_message_get_status(message)));
         return FALSE;
     }
 
-    content_size = soup_message_headers_get_content_length(message->response_headers);
+    content_size = soup_message_headers_get_content_length(soup_message_get_response_headers(message));
     content = g_malloc0(content_size);
 
     if (!g_input_stream_read_all(stream, content, content_size, NULL, NULL, &err)) {
         g_printerr("Could not load the content of %s: %s\n",
-                   LATEST_URI, err->message);
+                   from_url, err->message);
         return FALSE;
     }
 
@@ -275,7 +282,7 @@ static gboolean osinfo_db_get_latest_info(gchar **version,
 
     if (!json_parser_load_from_data(parser, content, -1, &err)) {
         g_printerr("Failed to parse the content of %s: %s\n",
-                   LATEST_URI, err->message);
+                   from_url, err->message);
         return FALSE;
     }
 
@@ -288,27 +295,29 @@ static gboolean osinfo_db_get_latest_info(gchar **version,
     if (!json_reader_read_member(reader, "release")) {
         const GError *error = json_reader_get_error(reader);
         g_printerr("Failed to read the \"release\" member of the %s file: %s\n",
-                   LATEST_URI, error->message);
+                   from_url, error->message);
         return FALSE;
     }
 
-    if (!json_reader_read_member(reader, "version")) {
+    if (version && !json_reader_read_member(reader, "version")) {
         const GError *error = json_reader_get_error(reader);
         g_printerr("Failed to read the \"version\" member of the %s file: %s\n",
-                   LATEST_URI, error->message);
+                   from_url, error->message);
         return FALSE;
     }
 
-    *version = g_strdup(json_reader_get_string_value(reader));
-    if (*version == NULL)
-        return FALSE;
+    if (version) {
+        *version = g_strdup(json_reader_get_string_value(reader));
+        if (*version == NULL)
+            return FALSE;
 
-    json_reader_end_member(reader); /* "version" */
+        json_reader_end_member(reader); /* "version" */
+    }
 
     if (!json_reader_read_member(reader, "archive")) {
         const GError *error = json_reader_get_error(reader);
         g_printerr("Failed to read the \"archive\" member of the %s file: %s\n",
-                   LATEST_URI, error->message);
+                   from_url, error->message);
         return FALSE;
     }
 
@@ -322,6 +331,17 @@ static gboolean osinfo_db_get_latest_info(gchar **version,
     return TRUE;
 }
 
+static gboolean osinfo_db_get_latest_info(gchar **version,
+                                          gchar **url)
+{
+    return osinfo_db_get_info(LATEST_URI, version, url);
+}
+
+static gboolean osinfo_db_get_nightly_info(gchar **url)
+{
+    return osinfo_db_get_info(NIGHTLY_URI, NULL, url);
+}
+
 static gboolean requires_soup(const gchar *source)
 {
     const gchar *prefixes[] = { "http://", "https://", NULL };
@@ -420,9 +440,10 @@ gint main(gint argc, gchar **argv)
     gboolean local = FALSE;
     gboolean system = FALSE;
     gboolean latest = FALSE;
+    gboolean nightly = FALSE;
     g_autofree gchar *installed_version = NULL;
     g_autofree gchar *latest_version = NULL;
-    g_autofree gchar *latest_url = NULL;
+    g_autofree gchar *archive_url = NULL;
     const gchar *root = "";
     const gchar *archive = NULL;
     const gchar *custom = NULL;
@@ -442,6 +463,8 @@ gint main(gint argc, gchar **argv)
         N_("Installation root directory"), NULL, },
       { "latest", 0, 0, G_OPTION_ARG_NONE, (void *)&latest,
         N_("Import the latest osinfo-db from osinfo-db's website"), NULL, },
+      { "nightly", 0, 0, G_OPTION_ARG_NONE, (void *)&nightly,
+        N_("Import the latest nightly build of unreleased osinfo-db from osinfo-db's website"), NULL, },
       { NULL, 0, 0, 0, NULL, NULL, NULL },
     };
     argv0 = argv[0];
@@ -468,6 +491,13 @@ gint main(gint argc, gchar **argv)
         return EXIT_FAILURE;
     }
 
+    /* check mutual exclusivity of --latest and --nightly */
+    if (latest && nightly)
+    {
+         g_printerr(_("--latest and --nightly are mutually exclusive\n"));
+         return EXIT_FAILURE;
+    }
+
     if (local)
         locs++;
     if (system)
@@ -484,18 +514,23 @@ gint main(gint argc, gchar **argv)
     archive = argc == 2 ? argv[1] : NULL;
     dir = osinfo_db_get_path(root, user, local, system, custom);
 
-    if (latest) {
+    if (nightly) {
+        if (!osinfo_db_get_nightly_info(&archive_url))
+            return EXIT_FAILURE;
+
+        archive = archive_url;
+    } else if (latest) {
         if (!osinfo_db_get_installed_version(dir, &installed_version))
             return EXIT_FAILURE;
 
-        if (!osinfo_db_get_latest_info(&latest_version, &latest_url))
+        if (!osinfo_db_get_latest_info(&latest_version, &archive_url))
             return EXIT_FAILURE;
 
         if (g_strcmp0(latest_version, installed_version) <= 0) {
             return EXIT_SUCCESS;
         }
 
-        archive = latest_url;
+        archive = archive_url;
     }
 
     if (osinfo_db_import_extract(dir, archive, verbose) < 0)
@@ -590,6 +625,16 @@ Downloads the latest osinfo-db release from libosinfo's official
 releases website and installs it in the desired location.
 The latest osinfo-db release is only downloaded and installed
 when it's newer than the one installed in the desired location.
+Note that this option is mutually exclusive with '--nightly'.
+
+=item B<--nightly>
+
+Downloads the nightly (unreleased) osinfo-db build from libosinfo's
+website and installs it in the desired location.
+Unlike with '--latest' with this option the nightly archive always
+downloaded and installed regardless of the version installed in the
+desired location. Note that this option is mutually exclusive with
+'--latest'.
 
 =item B<-v>, B<--verbose>
 
